//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: IR104OptoInput.c   v5.9
// Desc: Sample program that demonstrates how to perform opto inputs
// KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main IR104DSCIR104OptoInput
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
BYTE opto;   // relay number, 1-20
BYTE value;	 // value read from relay
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff; // temp variable of size int

int main( void )
{
	//=========================================================================
	// DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	//=========================================================================
	// BOARD INITIALIZATION
	//
	//	   Initialize the IR104 board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nIR104 BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf( "%hx", &dsccb.base_address );

	if(dscInitBoard(DSC_IR104, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	
	//////////////////////////////////////////////////
	// Opto Input
	// Reads opto value (whether it is high or low)
	//////////////////////////////////////////////////

	printf( "\nDIO INPUT AND OUTPUT:\n" );

	do
	{
		printf("Enter the opto number (1-20) : ");
		scanf("%d", &intBuff);
		opto = intBuff;

		getchar();

		if( ( result = dscIR104OptoInput( dscb, opto, &value )) != DE_NONE)
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscIR104OptoInput error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}

		printf( "Opto Number %d, Bit value received from opto: %d\n", (int) opto, (int) value );  
		printf("\nPress 'q' to exit or ENTER to continue \n");
	}while( getchar() != 'q' );
	
	

	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nDSCIR104OptoInput completed.\n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
